@@ -8,8 +8,10 @@ PARAMETER_USE_TRANSITION = "use_transition"
 DEFAULT_USE_TRANSITION = False
 PARAMETER_NO_SAMPLES = "no_samples"
 DEFAULT_MAX_NO_SAMPLES = 10000000
+PARAMETER_START = "start"
+PARAMETER_WINDOW_SIZE = "window_size"
 PARAMETER_NUM_RET_ITEMS = "num_ret_items"
-DEFAULT_MAX_NO_RET_ITEMS = 500
+DEFAULT_WINDOW_SIZE = 500
 
 KEYPHRASE = "hello"
 if str(os.name) == "posix":
