@@ -9,7 +9,8 @@ from pm4py.objects.log.util.xes import DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY,
 from pm4py.util import constants as pm4py_constants
 from pm4pydistr.configuration import PARAMETER_USE_TRANSITION, DEFAULT_USE_TRANSITION
 from pm4pydistr.configuration import PARAMETER_NO_SAMPLES, DEFAULT_MAX_NO_SAMPLES
-from pm4pydistr.configuration import PARAMETER_NUM_RET_ITEMS, DEFAULT_MAX_NO_RET_ITEMS
+from pm4pydistr.configuration import PARAMETER_NUM_RET_ITEMS, DEFAULT_WINDOW_SIZE, PARAMETER_START, \
+    PARAMETER_WINDOW_SIZE
 from pm4pydistr.log_handlers.parquet_filtering import factory as parquet_filtering_factory
 import pyarrow.parquet as pqq
 from pm4py.statistics.traces.pandas import case_statistics
@@ -21,6 +22,7 @@ from pathlib import Path
 PARQUET_CACHE = {}
 FILTERS = "filters"
 
+
 def get_columns_to_import(filters, columns, use_transition=False):
     if filters is None:
         filters = []
@@ -48,11 +50,14 @@ def get_columns_to_import(filters, columns, use_transition=False):
     columns = list(columns)
     return columns
 
+
 def insert_classifier(df):
     df["@@classifier"] = df[DEFAULT_NAME_KEY] + "+" + df[DEFAULT_TRANSITION_KEY]
     return df
 
-def load_parquet_from_path(path, columns, filters, use_transition=False, force_classifier_insertion=False, force_timestamp_conversion=False, parameters=None):
+
+def load_parquet_from_path(path, columns, filters, use_transition=False, force_classifier_insertion=False,
+                           force_timestamp_conversion=False, parameters=None):
     if parameters is None:
         parameters = {}
     if filters is None:
@@ -73,23 +78,28 @@ def load_parquet_from_path(path, columns, filters, use_transition=False, force_c
 
     return df
 
-def get_filtered_parquet(path, columns, filters, use_transition=False, force_classifier_insertion=False, parameters=None):
+
+def get_filtered_parquet(path, columns, filters, use_transition=False, force_classifier_insertion=False,
+                         parameters=None):
     if parameters is None:
         parameters = {}
     if filters is None:
         filters = []
     if columns is None:
         columns = []
-    if path in PARQUET_CACHE and not use_transition and set(columns).issubset(set([CASE_CONCEPT_NAME, DEFAULT_TIMESTAMP_KEY, DEFAULT_NAME_KEY])):
+    if path in PARQUET_CACHE and not use_transition and set(columns).issubset(
+            set([CASE_CONCEPT_NAME, DEFAULT_TIMESTAMP_KEY, DEFAULT_NAME_KEY])):
         df = PARQUET_CACHE[path]
     else:
-        df = load_parquet_from_path(path, columns, filters, use_transition=use_transition, force_classifier_insertion=force_classifier_insertion, parameters=parameters)
+        df = load_parquet_from_path(path, columns, filters, use_transition=use_transition,
+                                    force_classifier_insertion=force_classifier_insertion, parameters=parameters)
 
     if filters:
         df = parquet_filtering_factory.apply_filters(df, filters, parameters=parameters)
 
     return df
 
+
 def do_caching(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
@@ -104,18 +114,22 @@ def do_caching(path, log_name, managed_logs, parameters=None):
         if pq_basename in managed_logs:
             count = count + 1
 
-            df = load_parquet_from_path(pq, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY], [], use_transition=False, force_classifier_insertion=True, force_timestamp_conversion=True)
+            df = load_parquet_from_path(pq, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY], [],
+                                        use_transition=False, force_classifier_insertion=True,
+                                        force_timestamp_conversion=True)
             PARQUET_CACHE[pq] = df
 
             if count >= no_samples:
                 break
 
+
 def calculate_dfg(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -123,7 +137,8 @@ def calculate_dfg(path, log_name, managed_logs, parameters=None):
 
     if pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY in parameters:
         columns.append(parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY])
-        activity_key, parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY], activity_key
+        activity_key, parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = parameters[
+                                                                                         pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY], activity_key
     else:
         parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = activity_key
     folder = os.path.join(path, log_name)
@@ -137,7 +152,8 @@ def calculate_dfg(path, log_name, managed_logs, parameters=None):
             count = count + 1
             df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
             dfg = Counter(
-                df_statistics.get_dfg_graph(df, activity_key=activity_key, sort_timestamp_along_case_id=False, sort_caseid_required=False))
+                df_statistics.get_dfg_graph(df, activity_key=activity_key, sort_timestamp_along_case_id=False,
+                                            sort_caseid_required=False))
             overall_dfg = overall_dfg + dfg
             if count >= no_samples:
                 break
@@ -147,20 +163,24 @@ def calculate_dfg(path, log_name, managed_logs, parameters=None):
         returned_dict[el[0] + "@@" + el[1]] = overall_dfg[el]
     return returned_dict
 
+
 def calculate_performance_dfg(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
-    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY], use_transition=use_transition)
+    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY],
+                                    use_transition=use_transition)
 
     if pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY in parameters:
         columns.append(parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY])
-        activity_key, parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY], activity_key
+        activity_key, parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = parameters[
+                                                                                         pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY], activity_key
     else:
         parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = activity_key
     folder = os.path.join(path, log_name)
@@ -175,14 +195,17 @@ def calculate_performance_dfg(path, log_name, managed_logs, parameters=None):
             count = count + 1
             df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
 
-            f_dfg, p_dfg = df_statistics.get_dfg_graph(df, activity_key=activity_key, sort_timestamp_along_case_id=False, sort_caseid_required=False, measure="both")
+            f_dfg, p_dfg = df_statistics.get_dfg_graph(df, activity_key=activity_key,
+                                                       sort_timestamp_along_case_id=False, sort_caseid_required=False,
+                                                       measure="both")
             f_dfg = Counter(f_dfg)
 
             for k in p_dfg:
                 if k not in performance_dfg:
                     performance_dfg[k] = p_dfg[k]
                 else:
-                    performance_dfg[k] = (frequency_dfg[k] * performance_dfg[k] + f_dfg[k] * p_dfg[k])/(frequency_dfg[k] + f_dfg[k])
+                    performance_dfg[k] = (frequency_dfg[k] * performance_dfg[k] + f_dfg[k] * p_dfg[k]) / (
+                                frequency_dfg[k] + f_dfg[k])
 
             frequency_dfg = frequency_dfg + f_dfg
             if count >= no_samples:
@@ -193,24 +216,28 @@ def calculate_performance_dfg(path, log_name, managed_logs, parameters=None):
         returned_dict[el[0] + "@@" + el[1]] = performance_dfg[el]
     return returned_dict
 
+
 def calculate_process_schema_composite_object(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     performance_required = parameters["performance_required"] if "performance_required" in parameters else False
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
     if performance_required:
-        columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY], use_transition=use_transition)
+        columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY],
+                                        use_transition=use_transition)
     else:
         columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY], use_transition=use_transition)
 
     if pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY in parameters:
         columns.append(parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY])
-        activity_key, parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY], activity_key
+        activity_key, parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = parameters[
+                                                                                         pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY], activity_key
     else:
         parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = activity_key
     folder = os.path.join(path, log_name)
@@ -232,9 +259,12 @@ def calculate_process_schema_composite_object(path, log_name, managed_logs, para
             df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
 
             if performance_required:
-                f_dfg, p_dfg = df_statistics.get_dfg_graph(df, activity_key=activity_key, sort_timestamp_along_case_id=False, sort_caseid_required=False, measure="both")
+                f_dfg, p_dfg = df_statistics.get_dfg_graph(df, activity_key=activity_key,
+                                                           sort_timestamp_along_case_id=False,
+                                                           sort_caseid_required=False, measure="both")
             else:
-                f_dfg = df_statistics.get_dfg_graph(df, activity_key=activity_key, sort_timestamp_along_case_id=False, sort_caseid_required=False)
+                f_dfg = df_statistics.get_dfg_graph(df, activity_key=activity_key, sort_timestamp_along_case_id=False,
+                                                    sort_caseid_required=False)
 
             f_dfg = Counter(f_dfg)
 
@@ -244,7 +274,7 @@ def calculate_process_schema_composite_object(path, log_name, managed_logs, para
                         performance_dfg[k] = p_dfg[k]
                     else:
                         performance_dfg[k] = (frequency_dfg[k] * performance_dfg[k] + f_dfg[k] * p_dfg[k]) / (
-                                    frequency_dfg[k] + f_dfg[k])
+                                frequency_dfg[k] + f_dfg[k])
 
             frequency_dfg = frequency_dfg + f_dfg
             ea = Counter(end_activities_filter.get_end_activities(df, parameters=parameters))
@@ -285,12 +315,14 @@ def calculate_process_schema_composite_object(path, log_name, managed_logs, para
 
     return returned_dict
 
+
 def get_end_activities(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -326,7 +358,8 @@ def get_start_activities(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -360,7 +393,8 @@ def get_log_summary(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -393,12 +427,14 @@ def get_attribute_values(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
 
-    attribute_key = parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] if pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY in parameters else DEFAULT_NAME_KEY
+    attribute_key = parameters[
+        pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] if pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY in parameters else DEFAULT_NAME_KEY
 
     folder = os.path.join(path, log_name)
     columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, attribute_key], use_transition=use_transition)
@@ -444,7 +480,7 @@ def get_attribute_names(path, log_name, managed_logs, parameters=None):
                 break
 
     names = sorted(list(names))
-    names = [x.replace("AAA",":") for x in names]
+    names = [x.replace("AAA", ":") for x in names]
 
     return sorted(list(names))
 
@@ -454,14 +490,17 @@ def get_variants(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
-    no_ret_elements = parameters[PARAMETER_NUM_RET_ITEMS] if PARAMETER_NUM_RET_ITEMS in parameters else DEFAULT_MAX_NO_RET_ITEMS
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    window_size = parameters[PARAMETER_NUM_RET_ITEMS] if PARAMETER_NUM_RET_ITEMS in parameters else DEFAULT_WINDOW_SIZE
+    start = parameters[PARAMETER_START] if PARAMETER_START in parameters else 0
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
 
     folder = os.path.join(path, log_name)
-    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY], use_transition=use_transition)
+    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY],
+                                    use_transition=use_transition)
 
     parquet_list = parquet_importer.get_list_parquet(folder)
 
@@ -474,12 +513,13 @@ def get_variants(path, log_name, managed_logs, parameters=None):
         pq_basename = Path(pq).name
         if pq_basename in managed_logs:
             count = count + 1
-            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters, force_classifier_insertion=True)
+            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters,
+                                      force_classifier_insertion=True)
 
             events = events + len(df)
             cases = cases + df[CASE_CONCEPT_NAME].nunique()
 
-            #dictio = dictio + Counter(dict(df[attribute_key].value_counts()))
+            # dictio = dictio + Counter(dict(df[attribute_key].value_counts()))
             stats = case_statistics.get_variant_statistics(df)
             d_variants = {x["variant"]: x for x in stats}
 
@@ -492,7 +532,7 @@ def get_variants(path, log_name, managed_logs, parameters=None):
                     dictio_variants[variant]["count"] = dictio_variants[variant]["count"] + d_variants[variant]["count"]
 
             list_variants = sorted(list(dictio_variants.values()), key=lambda x: x["count"], reverse=True)
-            list_variants = list_variants[:min(len(list_variants), no_ret_elements)]
+            list_variants = list_variants[start:min(len(list_variants), window_size)]
             dictio_variants = {x["variant"]: x for x in list_variants}
 
             if count >= no_samples:
@@ -502,19 +542,23 @@ def get_variants(path, log_name, managed_logs, parameters=None):
 
     return {"variants": list_variants, "events": events, "cases": cases}
 
+
 def get_cases(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
-    no_ret_elements = parameters[PARAMETER_NUM_RET_ITEMS] if PARAMETER_NUM_RET_ITEMS in parameters else DEFAULT_MAX_NO_RET_ITEMS
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    window_size = parameters[PARAMETER_NUM_RET_ITEMS] if PARAMETER_NUM_RET_ITEMS in parameters else DEFAULT_WINDOW_SIZE
+    start = parameters[PARAMETER_START] if PARAMETER_START in parameters else 0
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
 
     folder = os.path.join(path, log_name)
-    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY], use_transition=use_transition)
+    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY],
+                                    use_transition=use_transition)
 
     parquet_list = parquet_importer.get_list_parquet(folder)
 
@@ -527,8 +571,8 @@ def get_cases(path, log_name, managed_logs, parameters=None):
         pq_basename = Path(pq).name
         if pq_basename in managed_logs:
             count = count + 1
-            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters, force_classifier_insertion=True)
-
+            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters,
+                                      force_classifier_insertion=True)
 
             events = events + len(df)
             cases = cases + df[CASE_CONCEPT_NAME].nunique()
@@ -536,10 +580,11 @@ def get_cases(path, log_name, managed_logs, parameters=None):
             stats = case_statistics.get_cases_description(df)
             c_list = []
             for x, y in stats.items():
-                c_list.append({"caseId": x, "caseDuration": y["caseDuration"], "startTime": y["startTime"], "endTime": y["endTime"]})
+                c_list.append({"caseId": x, "caseDuration": y["caseDuration"], "startTime": y["startTime"],
+                               "endTime": y["endTime"]})
 
             cases_list = sorted(cases_list + c_list, key=lambda x: x["caseDuration"], reverse=True)
-            cases_list = cases_list[:min(len(cases_list), no_ret_elements)]
+            cases_list = cases_list[start:min(len(cases_list), window_size)]
 
             if count >= no_samples:
                 break
@@ -552,7 +597,8 @@ def get_events(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -572,19 +618,19 @@ def get_events(path, log_name, managed_logs, parameters=None):
         if pq_basename in managed_logs:
             count = count + 1
 
-        df = get_filtered_parquet(pq, None, filters, use_transition=use_transition, parameters=parameters)
+            df = get_filtered_parquet(pq, None, filters, use_transition=use_transition, parameters=parameters)
 
-        try:
-            events = case_statistics.get_events(df, case_id)
-            if len(events) > 0:
-                df = parquet_importer.apply(pq)
-                ret = case_statistics.get_events(df, case_id)
+            try:
+                events = case_statistics.get_events(df, case_id)
+                if len(events) > 0:
+                    df = parquet_importer.apply(pq)
+                    ret = case_statistics.get_events(df, case_id)
+                    break
+            except:
+                pass
+
+            if count >= no_samples:
                 break
-        except:
-            pass
-
-        if count >= no_samples:
-            break
 
     return ret
 
@@ -594,7 +640,8 @@ def get_events_per_dotted(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -607,7 +654,8 @@ def get_events_per_dotted(path, log_name, managed_logs, parameters=None):
         attributes.append(parameters["attribute3"])
     attributes1 = list(set([CASE_CONCEPT_NAME] + [x for x in list(set(attributes)) if not x.startswith("@@")]))
 
-    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_TIMESTAMP_KEY] + attributes1, use_transition=use_transition)
+    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_TIMESTAMP_KEY] + attributes1,
+                                    use_transition=use_transition)
 
     folder = os.path.join(path, log_name)
 
@@ -676,7 +724,8 @@ def get_events_per_time(path, log_name, managed_logs, parameters=None):
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -697,16 +746,16 @@ def get_events_per_time(path, log_name, managed_logs, parameters=None):
         if pq_basename in managed_logs:
             count = count + 1
 
-        df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
+            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
 
-        if len(df) > max_no_of_points_to_sample:
-            df = df.sample(n=max_no_of_points_to_sample)
+            if len(df) > max_no_of_points_to_sample:
+                df = df.sample(n=max_no_of_points_to_sample)
 
-        date_values = [x.timestamp() for x in list(df[DEFAULT_TIMESTAMP_KEY])]
-        overall_list = overall_list + date_values
+            date_values = [x.timestamp() for x in list(df[DEFAULT_TIMESTAMP_KEY])]
+            overall_list = overall_list + date_values
 
-        if count >= no_samples:
-            break
+            if count >= no_samples:
+                break
 
     overall_list = sorted(overall_list)
     if len(overall_list) > max_no_of_points_to_sample:
@@ -715,12 +764,13 @@ def get_events_per_time(path, log_name, managed_logs, parameters=None):
     return overall_list
 
 
-def get_case_duration(path, log_name, managed_logs, parameters=None):
+def get_events_per_time_first(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
@@ -741,15 +791,17 @@ def get_case_duration(path, log_name, managed_logs, parameters=None):
         if pq_basename in managed_logs:
             count = count + 1
 
-        df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
+            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
+            df = df.groupby(CASE_CONCEPT_NAME).first()
 
-        cases = case_statistics.get_cases_description(df, parameters=parameters)
-        duration_values = [x["caseDuration"] for x in cases.values()]
+            if len(df) > max_no_of_points_to_sample:
+                df = df.sample(n=max_no_of_points_to_sample)
 
-        overall_list = overall_list + duration_values
+            date_values = [x.timestamp() for x in list(df[DEFAULT_TIMESTAMP_KEY])]
+            overall_list = overall_list + date_values
 
-        if count >= no_samples:
-            break
+            if count >= no_samples:
+                break
 
     overall_list = sorted(overall_list)
     if len(overall_list) > max_no_of_points_to_sample:
@@ -758,12 +810,91 @@ def get_case_duration(path, log_name, managed_logs, parameters=None):
     return overall_list
 
 
+def get_case_duration(path, log_name, managed_logs, parameters=None):
+    if parameters is None:
+        parameters = {}
+
+    no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
+    filters = parameters[FILTERS] if FILTERS in parameters else []
+    parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
+    parameters[pm4py_constants.PARAMETER_CONSTANT_ATTRIBUTE_KEY] = activity_key
+
+    max_no_of_points_to_sample = parameters[
+        "max_no_of_points_to_sample"] if "max_no_of_points_to_sample" in parameters else 100000
+
+    folder = os.path.join(path, log_name)
+    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME, DEFAULT_TIMESTAMP_KEY], use_transition=use_transition)
+
+    parquet_list = parquet_importer.get_list_parquet(folder)
+
+    overall_list = []
+    count = 0
+    for index, pq in enumerate(parquet_list):
+        pq_basename = Path(pq).name
+        if pq_basename in managed_logs:
+            count = count + 1
+
+            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
+
+            cases = case_statistics.get_cases_description(df, parameters=parameters)
+            duration_values = [x["caseDuration"] for x in cases.values()]
+
+            overall_list = overall_list + duration_values
+
+            if count >= no_samples:
+                break
+
+    overall_list = sorted(overall_list)
+    if len(overall_list) > max_no_of_points_to_sample:
+        overall_list = points_subset.pick_chosen_points_list(max_no_of_points_to_sample, overall_list)
+
+    return overall_list
+
+
+def get_events_per_case(path, log_name, managed_logs, parameters=None):
+    if parameters is None:
+        parameters = {}
+
+    no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    filters = parameters[FILTERS] if FILTERS in parameters else []
+
+    folder = os.path.join(path, log_name)
+    columns = get_columns_to_import(filters, [CASE_CONCEPT_NAME], use_transition=use_transition)
+
+    parquet_list = parquet_importer.get_list_parquet(folder)
+    length_counter = Counter()
+
+    count = 0
+    for index, pq in enumerate(parquet_list):
+        pq_basename = Path(pq).name
+        if pq_basename in managed_logs:
+            count = count + 1
+
+            df = get_filtered_parquet(pq, columns, filters, use_transition=use_transition, parameters=parameters)
+
+            case_dict = dict(df.groupby(CASE_CONCEPT_NAME).size())
+
+            for case, occ in case_dict.items():
+                length_counter[int(occ)] += 1
+
+            if count >= no_samples:
+                break
+
+    return dict(length_counter)
+
+
 def get_numeric_attribute_values(path, log_name, managed_logs, parameters=None):
     if parameters is None:
         parameters = {}
 
     no_samples = parameters[PARAMETER_NO_SAMPLES] if PARAMETER_NO_SAMPLES in parameters else DEFAULT_MAX_NO_SAMPLES
-    use_transition = parameters[PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
+    use_transition = parameters[
+        PARAMETER_USE_TRANSITION] if PARAMETER_USE_TRANSITION in parameters else DEFAULT_USE_TRANSITION
     activity_key = DEFAULT_NAME_KEY if not use_transition else "@@classifier"
     filters = parameters[FILTERS] if FILTERS in parameters else []
     parameters[pm4py_constants.PARAMETER_CONSTANT_ACTIVITY_KEY] = activity_key
